# RedXSecureUpdater

A comprehensive Python package for seamless and secure automatic project updates across different environments. RedXSecureUpdater provides real-time file monitoring, secure update distribution, and robust rollback capabilities.

## Installation

```bash
pip install RedXSecureUpdater
```

## Features

- **Automatic Updates**: Seamlessly update your application with a single command
- **Real-time File Monitoring**: Track changes in project files instantly
- **Safe Executable Updates**: Update running executables without crashes
- **Automatic Version Control**: Version management (e.g., 1.0.0 to 1.0.1)
- **Robust Rollback System**: Automatic backup and restore if updates fail
- **Update Validation**: Verify file integrity with checksums
- **Change Detection**: Track new, modified, and deleted files
- **Update Scheduling**: Configure automated update checks and installations
- **Compression Support**: Optimize update size with built-in compression
- **Update Notifications**: Keep users informed about update progress

## Quick Start Guide

### Basic Update Command

The simplest way to trigger updates is using the provided update trigger script:

```bash
# Basic version update
python -m redx_secure_updater.trigger -m "Update message" -v "1.1.0" -d "./my_app"

# Schedule update for specific time
python -m redx_secure_updater.trigger -m "Scheduled update" -v "1.2.0" -t "3:45 PM" -d "./my_app"

# Update with application restart
python -m redx_secure_updater.trigger -m "Update and restart" -v "2.0.0" -r "python my_app.py" -d "./my_app"
```

### Command Arguments Explained

- `-m` or `--message`: Update message to display (optional)
- `-v` or `--version`: New version number (e.g., "1.1.0")
- `-d` or `--dir`: Application directory to update
- `-t` or `--time`: Schedule time for the update (e.g., "3:45 PM")
- `-r` or `--restart`: Command to run after update completes

### How Updates Work

1. **Version Control**:
   - Maintains version history in `.redx/redx_state.json`
   - Validates version format (X.Y.Z)
   - Tracks file changes between versions

2. **Update Process**:
   ```python
   # Example: Update application to version 1.1.0
   from redx_secure_updater import RedXSecureUpdate

   result = RedXSecureUpdate(
       message="Bug fixes and improvements",
       version="1.1.0",
       directory="./my_app",
       app_command="python app.py"  # Optional restart command
   )
   ```

3. **Automatic Backup**:
   - Creates backup before applying updates
   - Stores backups in `.redx_backups`
   - Enables automatic rollback on failure

4. **File Monitoring**:
   ```python
   from redx_secure_updater import RedXSecureUpdater

   # Initialize updater with monitoring
   updater = RedXSecureUpdater(
       "your_app_directory",
       watch=True,  # Enable file monitoring
       compression_level=6  # Optional compression
   )

   # Start monitoring
   updater.start()
   ```

### Advanced Usage

#### Scheduled Updates

```python
# Schedule update for specific time
RedXSecureUpdate(
    message="Scheduled maintenance",
    version="2.0.0",
    schedule_time="3:45 PM",
    directory="./my_app"
)
```

#### Remote Updates

```python
# Setup remote update server
updater = RedXSecureUpdater(
    "your_app_directory",
    remote_url="https://your-update-server.com/updates",
    watch=True
)

# Configure update policies
updater.configure_scheduler(
    enabled=True,
    check_interval="1h",
    update_time="03:00",
    policies={
        "force_update": False,
        "notify_only": False,
        "backup_before_update": True,
        "auto_rollback": True
    }
)
```

### Best Practices

1. **Version Numbers**:
   - Use semantic versioning (MAJOR.MINOR.PATCH)
   - Example: 1.0.0, 1.1.0, 2.0.0

2. **Update Messages**:
   - Provide clear, descriptive messages
   - Include key changes and fixes

3. **Backup Strategy**:
   - Keep automated backups enabled
   - Test rollback functionality
   - Verify application state after updates

4. **Security**:
   - Use HTTPS for remote updates
   - Validate file checksums
   - Implement access controls

### Common Issues and Solutions

1. **Version Conflicts**:
   ```python
   # Fix: Explicitly set version
   RedXSecureUpdate(version="1.0.0", force=True)
   ```

2. **Update Failures**:
   - Check logs in `.redx/update_logs.txt`
   - Verify file permissions
   - Ensure no files are locked

3. **Scheduling Issues**:
   - Use 24-hour format for reliability
   - Consider timezone differences
   - Test scheduled updates

## Support

For issues and feature requests, please use our [GitHub issue tracker](https://github.com/redxdev/RedXSecureUpdater/issues).

## License

This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.